package dpexamples.day5.drawapp;

import dpexamples.day5.drawapp.model.*;

import java.awt.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.*;

public class DrawApplication extends MouseAdapter {

    private int x;
    private int y;
    private MouseEvent event;
    private DrawModel model = new DrawModel();
    private AbstractGraphic temporary;
    private JFrame frame = new JFrame();

    public DrawApplication() {
        JPanel outer = new JPanel(new BorderLayout());
        JComponent canvas = new DrawPanel();
        JToolBar toolBar = new JToolBar();
        toolBar.add(new JButton("Cut"));
        toolBar.add(new JButton("Copy"));
        toolBar.add(new JButton("Paste"));
        toolBar.add(new JButton("Undo"));
        toolBar.add(new JButton("Redo"));
        outer.add(toolBar, BorderLayout.PAGE_START);
        outer.add(canvas, BorderLayout.CENTER);

        canvas.addMouseListener(this);
        canvas.addMouseMotionListener(this);
        canvas.setBackground(Color.BLACK);
        canvas.setForeground(Color.WHITE);

        frame.getContentPane().add(outer);
        frame.setSize(640, 480);
        frame.setVisible(true);
    }

    public static void main(String[] args) {
        new DrawApplication();
    }

    public void mousePressed(MouseEvent e) {
        x = e.getX();
        y = e.getY();
    }

    public void mouseReleased(MouseEvent e) {
        event = null;
        if (temporary != null) {
            model.addGraphic(temporary.clone());
            temporary = null;
        }
        frame.repaint();
    }

    public void mouseDragged(MouseEvent e) {
        event = e;
        if (e.isAltDown()) {
            temporary = model.getPrototype(dpexamples.day5.drawapp.model.Box.class);
        } else if (e.isShiftDown()) {
            //moving = true;
        } else if (e.isControlDown()) {
            // temporary = model.getPrototype(Oval.class);
            // which should use: g.drawOval(g, Math.min(x1, x2), Math.min(y1, y2), width, height);
        } else {
            temporary = model.getPrototype(Line.class);
        }
        frame.repaint();
    }

    public void drawTemporary(Graphics2D g, MouseEvent e, int x, int y) {
        if (Math.abs(e.getX() - x) < 10 || Math.abs(e.getY() - y) < 10) {
            // ignore small drawings
            return;
        }

        if (temporary != null) {
            temporary.setCoordinates(x, y, e.getX(), e.getY());
            temporary.draw(g);
        }
    }

    private class DrawPanel extends JPanel {
        protected void paintComponent(Graphics g) {
            super.paintComponent(g);
            Graphics2D g2 = (Graphics2D) g;
            model.draw(g2);
            if (event != null) {
                g.setColor(Color.blue);
                drawTemporary(g2, event, x, y);
            }
        }
    }
}
